pub enum GenericListSource<'a> {
Anonymous,
Definition(&'a NameID),
ImplBlock {
target: &'a ImplTarget,
id: ImplID,
},
Expression(Loc<ExprID>),
}Variants
Anonymous
For when you just need a new generic list but have no need to refer back to it in the future
Definition(&'a NameID)
For managing the generics of callable with the specified name when type checking their body
ImplBlock
Expression(Loc<ExprID>)
For expressions which instantiate generic items