pub enum TypeVar {
Known(Loc<()>, KnownType, Vec<TypeVarID>),
Unknown(Loc<()>, u64, TraitList, MetaType),
}Expand description
A type variable represents the type of something in the program. It is mapped to expressions by type equations in the TypeState.
When TypeVars are passed externally into TypeState, they must be checked for replacement, as the type inference process might have refined them.
Variants
Known(Loc<()>, KnownType, Vec<TypeVarID>)
The base type is known and has a list of parameters
Unknown(Loc<()>, u64, TraitList, MetaType)
The type is unknown, but must satisfy the specified traits. When the generic substitution is done, the TypeVars will be carried over to the KnownType type vars
Implementations
Sourceimpl TypeVar
impl TypeVar
pub fn into_known(&self, type_state: &TypeState) -> Option<KnownTypeVar>
pub fn insert(self, into: &mut TypeState) -> TypeVarID
pub fn array(loc: Loc<()>, inner: TypeVarID, size: TypeVarID) -> Self
pub fn tuple(loc: Loc<()>, inner: Vec<TypeVarID>) -> Self
pub fn unit(loc: Loc<()>) -> Self
pub fn wire(loc: Loc<()>, inner: TypeVarID) -> Self
pub fn backward( loc: Loc<()>, inner: TypeVarID, type_state: &mut TypeState, ) -> Self
pub fn inverted(loc: Loc<()>, inner: TypeVarID) -> Self
pub fn expect_known<T, U, K, O>(&self, on_known: K, on_unknown: U) -> T
pub fn expect_named<T, E, U, K, O>( &self, on_named: K, on_unknown: U, on_other: O, on_error: E, ) -> T
Sourcepub fn resolve_named_or_inverted(
&self,
inverted_now: bool,
type_state: &TypeState,
) -> ResolvedNamedOrInverted
pub fn resolve_named_or_inverted( &self, inverted_now: bool, type_state: &TypeState, ) -> ResolvedNamedOrInverted
Expect a named type, or TypeVar::Inverted(named), or a recursive inversion. inverted_now is stateful and used to track if we are currently in an inverted context.
pub fn expect_specific_named<T, U, K, O>( &self, name: NameID, on_correct: K, on_unknown: U, on_other: O, ) -> T
Sourcepub fn expect_integer<T, E, U, K, O>(
&self,
on_integer: K,
on_unknown: U,
on_other: O,
on_error: E,
) -> T
pub fn expect_integer<T, E, U, K, O>( &self, on_integer: K, on_unknown: U, on_other: O, on_error: E, ) -> T
Assumes that this type is KnownType::Integer(size) and calls on_integer then. Otherwise calls on_unknown or on_other depending on the type. If the integer is given type params, panics
Sourcepub fn expect_string<T, E, U, K, O>(
&self,
on_string: K,
on_unknown: U,
on_other: O,
on_error: E,
) -> T
pub fn expect_string<T, E, U, K, O>( &self, on_string: K, on_unknown: U, on_other: O, on_error: E, ) -> T
Assumes that this type is KnownType::String(val) and calls on_string then. Otherwise calls on_unknown or on_other depending on the type. If the integer is given type params, panics
pub fn display(&self, type_state: &TypeState) -> String
pub fn display_with_meta( &self, display_meta: bool, type_state: &TypeState, ) -> String