pub struct TemplateTypeVarID { /* private fields */ }Expand description
A type which which should not be resolved directly but can be used to create new copies with unique type var ids
Implementations