Module method_resolution

Enums

FunctionLikeName

Traits

IntoImplTarget

Functions

select_method
Attempts to look up which function to call when calling method on a var of type self_type. Returns the method to call if it is fully known and exists, an error if there is no such method, or None if the method is ambiguous