Enums
Traits
Functions
- select_
method - Attempts to look up which function to call when calling
methodon a var of typeself_type. Returns the method to call if it is fully known and exists, an error if there is no such method, or None if the method is ambiguous